#include "rational.hpp"
#include <sstream>
#include <stdexcept>

Rational::Rational(int z, int n) : zaehler(z){
	if(n < 0){
		nenner = -n;
	}
	if(n > 0){
		nenner = n;
	}
	if(n == 0){
		throw std::invalid_argument("Nenner darf nicht 0 sein!");
	}
}

std::string Rational::toString() const {
	std::stringstream strstr;
	strstr << zaehler << "/" << nenner;
	return strstr.str();
}

Rational Rational::operator*(const Rational& rechts) const {
	Rational ergebnis(this->zaehler * rechts.zaehler, this->nenner * rechts.nenner);
	return ergebnis;
}

Rational& Rational::operator*=(const Rational& rechts) {
	this->zaehler *= rechts.zaehler;
	this->nenner *= rechts.nenner;
	return *this;
}

Rational Rational::operator+(const Rational& rechts) const {
	int neuNenner = Rational::kgV( this->nenner, rechts.nenner);

	int z = neuNenner/this->nenner;
	int zR = neuNenner/rechts.nenner;
	
	int neuZaehler = (this->zaehler * z) + (rechts.zaehler * zR);

	Rational ergebnis(neuZaehler, neuNenner);
	return ergebnis;
}

Rational& Rational::operator+=(const Rational& rechts) {
  	int neuNenner = Rational::kgV(this->nenner, rechts.nenner);
	
	int z = neuNenner/this->nenner;
	int zR = neuNenner/rechts.nenner;
	
	this->zaehler = (this->zaehler * z) + (rechts.zaehler * zR);
	this->nenner = neuNenner;
	
  return *this;
}

Rational Rational::operator-(const Rational& rechts) const{     
    	int neuNenner = Rational::kgV( this->nenner, rechts.nenner);

	int z = neuNenner/this->nenner;
	int zR = neuNenner/rechts.nenner;
	
	int neuZaehler = (this->zaehler * z) - (rechts.zaehler * zR);

	Rational ergebnis(neuZaehler, neuNenner);
	return ergebnis;
}

Rational& Rational::operator-=(const Rational& rechts) {
   	int neuNenner = Rational::kgV(this->nenner, rechts.nenner);
	
	int z = neuNenner/this->nenner;
	int zR = neuNenner/rechts.nenner;
	
	this->zaehler = (this->zaehler * z) - (rechts.zaehler * zR);
	this->nenner = neuNenner;
  	return *this;
}

Rational Rational::operator/(const Rational& rechts) const{   
	int neuZaehler = this->zaehler * rechts.nenner;
	int neuNenner = this->nenner * rechts.zaehler;
	return Rational(neuZaehler, neuNenner);
}

Rational& Rational::operator/=(const Rational& rechts) {
	if(rechts.zaehler == 0)
	{
		throw std::invalid_argument("Nenner wird zu 0!");
	} 
    	this->zaehler = this->zaehler * rechts.nenner;
	this->nenner = this->nenner * rechts.zaehler;
	
  	return *this;
}

bool Rational::operator==(const Rational& rechts) const{  
         if((this->zaehler * rechts.nenner) == (rechts.zaehler * this->nenner))
	{
		return true;
	}

	return false;
}

bool Rational::operator!=(const Rational& rechts) const{
  
   if((this->zaehler * rechts.nenner) != (rechts.zaehler * this->nenner))
	{
		return true;
	}

	return false;
}

bool Rational::operator<(const Rational& rechts) const{  
	 if((this->zaehler * rechts.nenner) < (rechts.zaehler * this->nenner))
	{
		return true;
	}

	return false;
}

bool Rational::operator<=(const Rational& rechts) const{
   if((this->zaehler * rechts.nenner) <= (rechts.zaehler * this->nenner))
	{
		return true;
	}

	return false;
}

bool Rational::operator>(const Rational& rechts) const{     
	 if((this->zaehler * rechts.nenner) > (rechts.zaehler * this->nenner))
	{
		return true;
	}

	return false;
}

bool Rational::operator>=(const Rational& rechts) const{
   if((this->zaehler * rechts.nenner) >= (rechts.zaehler * this->nenner))
	{
		return true;
	}

	return false;
}


std::ostream& operator<<(std::ostream& os, const Rational& r){
	os << r.toString();
	return os;
}

int Rational::kgV(int a, int b){
    int result;
    if(a < b){
         result = b;
	}
	else{ result = a;
	a = b;
	b = result;
	}
    while(result % a != 0)
    {
                 result+= b;
    }
    return result;
}




